#!/usr/bin/env python
###############################################################################
#              a1q7_ASThread_15505623484889_6592_29
###############################################################################
#
#SBATCH -J a1q7_ASThread_15505623484889_6592_29
#
#SBATCH -A pr1ehu00
#SBATCH -o a1q7_ASThread_15505623484889_6592_29.out
#SBATCH -e a1q7_ASThread_15505623484889_6592_29.err
#SBATCH -t 02:00:00
#SBATCH -n 6592
#

#
###############################################################################

import os
import sys
from threading import Thread
from commands import getstatusoutput
from datetime import datetime
from math import ceil
from collections import OrderedDict
import copy

# Defining scripts to be run
scripts = [['a1q7_20120203_1_DA.cmd', 'a1q7_20120924_000_1_SIM.cmd', 'a1q7_20120924_001_1_SIM.cmd', 'a1q7_20120924_002_1_SIM.cmd', 'a1q7_20120924_003_1_SIM.cmd', 'a1q7_20120924_004_1_SIM.cmd', 'a1q7_20120924_005_1_SIM.cmd', 'a1q7_20120924_006_1_SIM.cmd', 'a1q7_20120924_007_1_SIM.cmd', 'a1q7_20120924_008_1_SIM.cmd', 'a1q7_20120924_009_1_SIM.cmd', 'a1q7_20120924_010_1_SIM.cmd', 'a1q7_20120924_011_1_SIM.cmd'], ['a1q7_20120924_1_DA.cmd', 'a1q7_20120203_005_2_SIM.cmd', 'a1q7_20120203_001_2_SIM.cmd', 'a1q7_20120203_006_2_SIM.cmd', 'a1q7_20120203_007_2_SIM.cmd', 'a1q7_20120203_004_2_SIM.cmd', 'a1q7_20120203_000_2_SIM.cmd', 'a1q7_20120203_008_2_SIM.cmd', 'a1q7_20120203_009_2_SIM.cmd', 'a1q7_20120203_010_2_SIM.cmd', 'a1q7_20120203_003_2_SIM.cmd', 'a1q7_20120203_011_2_SIM.cmd', 'a1q7_20120203_002_2_SIM.cmd'], ['a1q7_20120924_000_2_SIM.cmd', 'a1q7_20120924_006_2_SIM.cmd'], ['a1q7_20120924_2_DA.cmd']]

class JobThread(Thread):
    def __init__ (self, template, id_run):
        Thread.__init__(self)
        self.template = template
        self.id_run = id_run

    def run(self):
        jobname = self.template.replace('.cmd', '')
        os.system("echo $(date +%s) > "+jobname+"_STAT")
        out = str(self.template) + "." + str(self.id_run) + ".out"
        err = str(self.template) + "." + str(self.id_run) + ".err"
        command = "bash " + str(self.template) + " " + str(self.id_run) + " " + os.getcwd()
        (self.status) = getstatusoutput(command + " > " + out + " 2> " + err)

class JobListThread(Thread):
    def __init__ (self, jobs_list, id_run, all_cores):
        Thread.__init__(self)
        self.jobs_list = jobs_list
        self.id_run = id_run
        self.all_cores = all_cores

    def run(self):
        all_cores = self.all_cores
                
        pid_list = []
        
        for i in range(len(self.jobs_list)):
            if type(self.jobs_list[i]) != list:
                job = self.jobs_list[i]
                jobname = job.replace(".cmd", '')
                section = jobname.split('_')[-1]
        
                    
                machines = str()
                
                cores = int(jobs_resources[section]['PROCESSORS'])
                tasks = int(jobs_resources[section]['TASKS'])
                nodes = int(ceil(int(cores)/float(tasks)))
                if tasks < processors_per_node:
                    cores = tasks
                
                job_cores = cores
                while nodes > 0:
                    while cores > 0:
                        if len(all_cores) > 0:
                            node = all_cores.pop(0)
                            if node:
                                machines += node +"\n"
                                cores -= 1
                    for rest in range(processors_per_node-tasks):
                        if len(all_cores) > 0:
                            all_cores.pop(0)
                    nodes -= 1
                    if tasks < processors_per_node:
                        cores = job_cores
                
                machines = "\n".join([s for s in machines.split("\n") if s])
                with open("machinefiles/machinefile_"+jobname, "w") as machinefile:
                    machinefile.write(machines)
        
        
            current = JobThread(self.jobs_list[i], i)
            pid_list.append(current)
            current.start()
        
        # Waiting until all scripts finish
        for i in range(len(pid_list)):
            pid = pid_list[i]
            pid.join()
            
            completed_filename = self.jobs_list[i].replace('.cmd', '_COMPLETED')
            completed_path = os.path.join(os.getcwd(), completed_filename)
            if os.path.exists(completed_path):
                print datetime.now(), "The job ", pid.template," has been COMPLETED"
            else:
                print datetime.now(), "The job ", pid.template," has FAILED"
                os._exit(1)


# Getting the list of allocated nodes
os.system("scontrol show hostnames $SLURM_JOB_NODELIST > node_list")
os.system("mkdir -p machinefiles")

with open('node_list', 'r') as file:
     all_nodes = file.read()

all_nodes = all_nodes.split("\n")

total_cores = 6592
jobs_resources = {'MACHINEFILES': 'STANDARD', 'PROCESSORS_PER_NODE': '48', 'SIM': {'TASKS': '48', 'PROCESSORS': '528'}, 'DA': {'TASKS': '48', 'PROCESSORS': '256'}}

processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])

idx = 0
all_cores = []
while total_cores > 0:
    if processors_per_node > 0:
        processors_per_node -= 1
        total_cores -= 1
        all_cores.append(all_nodes[idx])
    else:
        idx += 1
        processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])

processors_per_node = int(jobs_resources['PROCESSORS_PER_NODE'])

for i in range(len(scripts)):
    current = JobListThread(scripts[i], i, copy.deepcopy(all_cores))
    current.start()
    current.join()
